#ifndef MSP_GL_COLORCURVE_H_
#define MSP_GL_COLORCURVE_H_

#include "postprocessor.h"
#include "program.h"
#include "programdata.h"
#include "texture1d.h"
#include "texturing.h"

namespace Msp {
namespace GL {

/**
Processes oversaturated colors to preserve hues.  When one color component
exceeds 1.0, the overflow is distributed to the other components, scaling the
color towards white.

Gamma or sRGB correction can also be applied to the output.  It can be used to
improve color reproduction by performing lighting calculations in linear color
space and converting to sRGB for display.
*/
class ColorCurve: public PostProcessor
{
public:
	struct Template: public PostProcessor::Template
	{
		class Loader: public DataFile::DerivedObjectLoader<Template, PostProcessor::Template::Loader>
		{
		public:
			Loader(Template &);

		private:
			void gamma(float);
			void srgb();
		};

		float exposure_adjust;
		float brightness_response;
		float gamma;
		bool srgb;

		Template();

		virtual ColorCurve *create(unsigned, unsigned) const;
	};

private:
	Program shprog;
	ProgramData shdata;
	Texture1D curve;
	Texturing texturing;
	const Mesh &quad;

public:
	ColorCurve();

	/** Set exposure adjustment in EV units.  Positive values brighten the
	image, negative values darken it.  Zero is neutral. */
	void set_exposure_adjust(float);

	/** Sets the exponent of the */
	void set_brightness_response(float);

	// Deprecated functions
	void set_peak(float);
	void set_brightness(float);

	/** Sets the gamma value used for mapping output colors.  Allowed range is
	from 0.1 to 10. */
	void set_gamma(float);

	/** Sets output mapping to sRGB.  This is almost, but not exactly equivalent
	to set_gamma(2.2). */
	void set_srgb();

	/// Sets output mapping to linear.  This is equivalent to set_gamma(1).
	void set_linear();

	virtual void render(Renderer &, const Texture2D &, const Texture2D &);
};

} // namespace GL
} // namespace Msp

#endif
